/*
  Copyright (c) 2007 Thomas Jahns <Thomas.Jahns@gmx.net>

  Permission to use, copy, modify, and distribute this software for any
  purpose with or without fee is hereby granted, provided that the above
  copyright notice and this permission notice appear in all copies.

  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
  WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
  MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
  ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
  ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

#include "libgtcore/env.h"
#include "libgtcore/option.h"
#include "libgtmatch/eis-bwtseq.h"

#include "gt_packedindex_blockenc_params.h"
#include "gt_packedindex_bwtconstruct_params.h"

extern void
registerPackedIndexOptions(OptionParser *op, struct bwtOptions *paramOutput,
                           int defaultOptimizationFlags,
                           const Str *projectName, Env *env)
{
  Option *option;

  env_error_check(env);

  registerBlockEncOptions(op, &paramOutput->final.seqParams.blockEnc,
                          env);

  paramOutput->final.baseType = BWT_ON_BLOCK_ENC;

  paramOutput->final.bwtFeatureToggles = BWTLocateBitmap;

  option = option_new_uint(
    "locfreq", "specify the locate frequency\n"
    "parameter i means that each i-th position of input string is stored\n"
    "0 => no locate information", &paramOutput->final.locateInterval, 16U, env);
  option_parser_add_option(op, option, env);

  paramOutput->final.projectName = projectName;

  paramOutput->defaultOptimizationFlags = defaultOptimizationFlags;
}

extern void
computePackedIndexDefaults(struct bwtOptions *paramOutput, Env *env)
{
  switch (paramOutput->final.baseType)
  {
  case BWT_ON_BLOCK_ENC:
    {
      int *EISFeatureSet = &paramOutput->final.seqParams.blockEnc.EISFeatureSet;
      *EISFeatureSet = EIS_FEATURE_NONE;
      if (paramOutput->defaultOptimizationFlags & BWTDEFOPT_LOW_RAM_OVERHEAD)
      {
        *EISFeatureSet &= ~EIS_FEATURE_REGION_SUMS;
      }
      if (paramOutput->defaultOptimizationFlags & BWTDEFOPT_FAST_RANK)
      {
        *EISFeatureSet |= EIS_FEATURE_REGION_SUMS;
      }
    }
    break;
  default:
    break;
  }
}
